"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _nullishCoalesce(lhs, rhsFn) { if (lhs != null) { return lhs; } else { return rhsFn(); } }// src/index.ts
var _unplugin = require('unplugin');
var _pluginutils = require('@rollup/pluginutils');
var _common = require('@vue-macros/common');

// src/core/index.ts









// ../../node_modules/.pnpm/estree-walker@3.0.1/node_modules/estree-walker/src/walker.js
var WalkerBase = class {
  constructor() {
    this.should_skip = false;
    this.should_remove = false;
    this.replacement = null;
    this.context = {
      skip: () => this.should_skip = true,
      remove: () => this.should_remove = true,
      replace: (node) => this.replacement = node
    };
  }
  replace(parent, prop, index, node) {
    if (parent) {
      if (index !== null) {
        parent[prop][index] = node;
      } else {
        parent[prop] = node;
      }
    }
  }
  remove(parent, prop, index) {
    if (parent) {
      if (index !== null) {
        parent[prop].splice(index, 1);
      } else {
        delete parent[prop];
      }
    }
  }
};

// ../../node_modules/.pnpm/estree-walker@3.0.1/node_modules/estree-walker/src/sync.js
var SyncWalker = class extends WalkerBase {
  constructor(enter, leave) {
    super();
    this.enter = enter;
    this.leave = leave;
  }
  visit(node, parent, prop, index) {
    if (node) {
      if (this.enter) {
        const _should_skip = this.should_skip;
        const _should_remove = this.should_remove;
        const _replacement = this.replacement;
        this.should_skip = false;
        this.should_remove = false;
        this.replacement = null;
        this.enter.call(this.context, node, parent, prop, index);
        if (this.replacement) {
          node = this.replacement;
          this.replace(parent, prop, index, node);
        }
        if (this.should_remove) {
          this.remove(parent, prop, index);
        }
        const skipped = this.should_skip;
        const removed = this.should_remove;
        this.should_skip = _should_skip;
        this.should_remove = _should_remove;
        this.replacement = _replacement;
        if (skipped)
          return node;
        if (removed)
          return null;
      }
      for (const key in node) {
        const value = node[key];
        if (typeof value !== "object") {
          continue;
        } else if (Array.isArray(value)) {
          for (let i = 0; i < value.length; i += 1) {
            if (value[i] !== null && typeof value[i].type === "string") {
              if (!this.visit(value[i], node, key, i)) {
                i--;
              }
            }
          }
        } else if (value !== null && typeof value.type === "string") {
          this.visit(value, node, key, null);
        }
      }
      if (this.leave) {
        const _replacement = this.replacement;
        const _should_remove = this.should_remove;
        this.replacement = null;
        this.should_remove = false;
        this.leave.call(this.context, node, parent, prop, index);
        if (this.replacement) {
          node = this.replacement;
          this.replace(parent, prop, index, node);
        }
        if (this.should_remove) {
          this.remove(parent, prop, index);
        }
        const removed = this.should_remove;
        this.replacement = _replacement;
        this.should_remove = _should_remove;
        if (removed)
          return null;
      }
    }
    return node;
  }
};

// ../../node_modules/.pnpm/estree-walker@3.0.1/node_modules/estree-walker/src/index.js
function walk(ast, { enter, leave }) {
  const instance = new SyncWalker(enter, leave);
  return instance.visit(ast, null);
}

// src/core/index.ts


// src/core/constants.ts
var SETUP_COMPONENT_ID_SUFFIX = "/setup-component/";
var SETUP_COMPONENT_ID_REGEX = /\/setup-component\/(\d+).vue$/;
var SETUP_COMPONENT_SUB_MODULE = /\/setup-component\/(\d+).vue/;
var SETUP_COMPONENT_TYPE = "SetupFC";

// src/core/index.ts
var scanSetupComponent = (code, id) => {
  const program = _common.babelParse.call(void 0, code, _common.getLang.call(void 0, id));
  const nodes = [];
  walk(program, {
    enter(node) {
      var _a;
      if (_common.isCallOf.call(void 0, node, _common.DEFINE_SETUP_COMPONENT)) {
        nodes.push({ fn: node.arguments[0], src: node });
      } else if (node.type === "VariableDeclarator" && node.id.type === "Identifier" && ((_a = node.id.typeAnnotation) == null ? void 0 : _a.type) === "TSTypeAnnotation" && node.id.typeAnnotation.typeAnnotation.type === "TSTypeReference" && node.id.typeAnnotation.typeAnnotation.typeName.type === "Identifier" && node.id.typeAnnotation.typeAnnotation.typeName.name === SETUP_COMPONENT_TYPE && node.init) {
        nodes.push({ fn: node.init });
      }
    }
  });
  if (nodes.length === 0)
    return [];
  return nodes.map(({ fn, src }) => {
    if (!["FunctionExpression", "ArrowFunctionExpression"].includes(fn.type))
      throw new SyntaxError(
        `${_common.DEFINE_SETUP_COMPONENT}: invalid setup component definition`
      );
    const body = fn.body;
    let bodyStart = body.start;
    let bodyEnd = body.end;
    if (body.type === "BlockStatement") {
      bodyStart++;
      bodyEnd--;
    }
    return {
      code: code.slice(fn.start, fn.end),
      body: code.slice(bodyStart, bodyEnd),
      node: src || fn
    };
  });
};
var transformSetupComponent = (code, id, ctx) => {
  const normalizedId = _pluginutils.normalizePath.call(void 0, id);
  const s = new (0, _common.MagicString)(code);
  const nodeContexts = scanSetupComponent(code, id);
  ctx[normalizedId] = nodeContexts;
  for (const [i, { node }] of nodeContexts.entries()) {
    const importName = `setupComponent_${i}`;
    s.overwrite(node.start, node.end, importName);
    s.prepend(
      `import ${importName} from '${normalizedId}${SETUP_COMPONENT_ID_SUFFIX}${i}.vue'
`
    );
  }
  return _common.getTransformResult.call(void 0, s, id);
};
var loadSetupComponent = (virtualId, ctx, root) => {
  var _a, _b, _c;
  const index = +(_nullishCoalesce(((_a = SETUP_COMPONENT_ID_REGEX.exec(virtualId)) == null ? void 0 : _a[1]), () => ( -1)));
  const id = virtualId.replace(SETUP_COMPONENT_ID_REGEX, "");
  const nodeCtx = ((_b = ctx[id]) == null ? void 0 : _b[index]) || ((_c = ctx[root + id]) == null ? void 0 : _c[index]);
  if (!nodeCtx)
    return;
  const { body } = nodeCtx;
  const lang = _common.getLang.call(void 0, id);
  const program = _common.babelParse.call(void 0, body, lang, { allowReturnOutsideFunction: true });
  const s = new (0, _common.MagicString)(body);
  for (const stmt of program.body) {
    if (stmt.type !== "ReturnStatement" || !stmt.argument)
      continue;
    s.overwriteNode(stmt, `defineRender(${s.sliceNode(stmt.argument)});`);
  }
  s.prepend(`<script setup${lang ? ` lang="${lang}"` : ""}>`);
  s.append(`<\/script>`);
  return s.toString();
};
var hotUpdateSetupComponent = async ({ file, modules, read }, ctx) => {
  const isSubModule = (id) => SETUP_COMPONENT_SUB_MODULE.test(id);
  const getSubModule = (module2) => {
    const importedModules = Array.from(module2.importedModules);
    if (importedModules.length === 0)
      return [];
    return importedModules.filter(({ id }) => id && isSubModule(id)).flatMap((module3) => {
      return [module3, ...getSubModule(module3)];
    });
  };
  const module = modules.find((mod) => mod.file === file);
  if (!module || !module.id)
    return;
  const affectedModules = getSubModule(module);
  const normalizedId = _pluginutils.normalizePath.call(void 0, file);
  const nodeContexts = scanSetupComponent(await read(), normalizedId);
  ctx[normalizedId] = nodeContexts;
  return [...modules, ...affectedModules];
};

// src/index.ts
function resolveOption(options) {
  return {
    include: [_common.REGEX_SRC_FILE],
    exclude: [_common.REGEX_SETUP_SFC],
    root: process.cwd(),
    ...options
  };
}
var name = "unplugin-vue-setup-component";
var src_default = _unplugin.createUnplugin.call(void 0, (userOptions = {}) => {
  const options = resolveOption(userOptions);
  const filter = _pluginutils.createFilter.call(void 0, options.include, options.exclude);
  const setupComponentContext = {};
  return {
    name,
    enforce: "pre",
    resolveId(id) {
      if (SETUP_COMPONENT_ID_REGEX.test(id))
        return id;
    },
    loadInclude(id) {
      return SETUP_COMPONENT_ID_REGEX.test(id);
    },
    load(id) {
      return loadSetupComponent(id, setupComponentContext, options.root);
    },
    transformInclude(id) {
      return filter(id);
    },
    transform(code, id) {
      try {
        return transformSetupComponent(code, id, setupComponentContext);
      } catch (err) {
        this.error(`${name} ${err}`);
      }
    },
    vite: {
      configResolved(config) {
        options.root = config.root;
      },
      handleHotUpdate: (ctx) => {
        if (filter(ctx.file)) {
          return hotUpdateSetupComponent(ctx, setupComponentContext);
        }
      }
    }
  };
});



exports.src_default = src_default;
